"""
Snakefile for

Author: Jeffrey Pullin
Affiliation: University of Melbourne

Run:

* In RStudio: snakemake -s Snakefile.py --cores 2 --rerun-incomplete
* On server: snakemake -s Snakefile.py --profile slurm --use-conda

Jeffrey Pullin 2020/2021
"""

import json

configfile: "config.yaml"
R = config["R"]

sim_pars = glob_wildcards("config/sim_pars/{sim_par}.json").sim_par
method_pars = glob_wildcards("config/method_pars/{method_par}.json").method_par

sim_dirs = expand(
  config["sim_data_folder"] + "{sim_par}.rds",
  sim_par = sim_pars
)

res_dirs = expand(
  config["results_folder"] + "{sim_par}-{method_par}.rds", 
  method_par = method_pars,
  sim_par = sim_pars
)

mg_dirs = expand(
  config["sim_data_folder"] + "{mg_type}-{sim_par}.rds",
  sim_par = sim_pars, 
  mg_type = config["mg_type"]
)

shell("module load r/4.0.0")
shell("module load snakemake/5.18.1")

rule all:
  input: sim_dirs, res_dirs, mg_dirs

rule simulate_data:
  priority: 100
	input:  script = config["code_folder"] + "simulate-data.R",
			sim_pars = config["sim_pars_folder"] + "{sim_id}.json",
	output: sim = config["sim_data_folder"] + "{sim_id}.rds"
	log:	config["logs_folder"] + "sim_data-{sim_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: attempt * 6000,
	  time_min=lambda wildcards, attempt: [30, 60, 120][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={output.sim}\
		sim_pars={input.sim_pars} wcs={wildcards}"\
		{input.script} {log}'''

rule extract_marker_genes:
	input:  script = config["code_folder"] + "extract-marker-genes.R",
	    sim = config["sim_data_folder"] + "{sim_id}.rds",
	output: mgs = config["sim_data_folder"] + "{mg_type}-{sim_id}.rds"
	log:	config["logs_folder"] + "extract_mgs-{mg_type}-{sim_id}.Rout"
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={input.sim} mg_type={wildcards.mg_type}\
		mgs={output.mgs}"\
		{input.script} {log}'''

rule run_method:
	priority: 99
	input:	script = config["code_folder"] + "run-method.R",
			sim = config["sim_data_folder"] + "{sim_id}.rds",
			method_pars = config["method_pars_folder"] + "{method_id}.json",
			fun = lambda wc: config["code_folder"] + "run-" + wc.method_id.split('_')[0] + ".R"
	output: res = config["results_folder"] + "{sim_id}-{method_id}.rds"
	log:	config["logs_folder"] + "run_method-{sim_id}-{method_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 3000,
	  time_min=lambda wildcards, attempt: [20, 60, 360][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={input.sim} fun={input.fun}\
		meth_pars={input.method_pars} res={output.res}"\
		{input.script} {log}'''

onsuccess:
    print("Success! The Snakemake workflow is completed.")

onerror:
    print("Error! The Snakemake workflow aborted.")
