# This file contains code to simulate the single cell RNA seq data sets for use
# in the downstream computational experiments.

suppressMessages({
  library(SingleCellExperiment)
  library(scran)
  library(scater)
  library(splatter)
})

# Functions

source(here::here("code", "utils.R"))

# Code

# Get the passed arguments from the command line via Snakemake.
args <- R.utils::commandArgs(trailingOnly = TRUE, asValues = TRUE)

# Debugging.
print(args)

# Get the simulation parameters.
sim_pars <- jsonlite::fromJSON(args$sim_pars)
wcs <- parse_wildcards(args$wcs)

# Debugging.
print(wcs)

print(sim_pars$seed)

set.seed(sim_pars$seed)

sim_data <- readRDS(paste0(here::here("data", sim_pars$data_id), ".rds"))
# Spatter cannot handle sparse matrices (or DelayedArray objects).
sim_counts <- as.matrix(counts(sim_data))
params <- splatEstimate(sim_counts)

# We need to re-weight the probabilities as small numerical issues creep in from
# storing the parameters in json and Splatter is strict about checking they
# sum to 1.
group_prob <- fix_prob(sim_pars$group.prob)

params <- setParam(params, "group.prob", group_prob)
params <- setParam(params, "batchCells", sim_pars$batchCells)
params <- setParam(params, "seed", sim_pars$seed)
params <- setParam(params, "de.facLoc", sim_pars$de.facLoc)

if (!is.null(sim_pars$mean.shape)) {
  params <- setParam(params, "mean.shape", sim_pars$mean.shape)
}

if (!is.null(sim_pars$mean.rate)) {
  params <- setParam(params, "mean.rate", sim_pars$mean.shape)
}

Sys.time()

# Run the splatter simulation.
sim <- splatSimulateGroups(params)

Sys.time()

# Construct the log normalized counts needed for downstream analysis.
# TODO: Should we use deconvolution or just the naive size factors?
clusters <- quickCluster(sim)
sim <- computeSumFactors(sim, clusters = clusters)
sim <- logNormCounts(sim)

Sys.time()

# Perform dimensionality reduction.
sim <- runPCA(sim)
sim <- runTSNE(sim, dimred = "PCA")
sim <- runUMAP(sim, dimred = "PCA")

Sys.time()

# Label the columns with the true simulated groups.
colLabels(sim) <- colData(sim)$Group

# Debugging.
print(sim)

saveRDS(sim, args$sim)
