# Produce the simulation parameters for all simulations
config <- yaml::read_yaml(here::here("config.yaml"))

suppressMessages({
  library(tibble)
  library(tidyr)
  library(dplyr)
  library(purrr)
})

o <- options(scipen = 10)
on.exit(options(scipen = o$scipen), add = TRUE)

# A 'standard' simulation meant to reflect a most plausible scenario.
standard_sims <- expand_grid(
  label = "standard_sim",
  data_id = "lawlor",
  de.facLoc = 2,
  rep = 1:2,
  batchCells = 2000,
  n_clus = 5,
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column %>%
  mutate(
    seed = 1000 + rowid,
    file_name = paste0(label, "_", rep)
  )

# A 'standard' simulation meant to reflect a most plausible scenario.
mean_sims <- expand_grid(
  label = "mean_sim",
  de.facLoc = 2,
  rep = 1:2,
  batchCells = 2000,
  n_clus = 5,
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column %>%
  mutate(
    seed = 1000 + rowid,
    file_name = paste0(label, "_", rep)
  )

# Simulations without DE.
null_sims <- expand_grid(
  label = "null_sim",
  data_id = "lawlor",
  de.facLoc = 0.1,
  rep = 1:2,
  batchCells = 2000,
  n_clus = 5,
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column() %>%
  mutate(
    seed = 2000 + rowid,
    file_name = paste0(label, "_", rep)
  )

# Changing total number of cells.
num_cells_sims <- expand_grid(
  label = "num_cells",
  data_id = "lawlor",
  de.facLoc = 2,
  rep = 1:3,
  n_clus = 5,
  batchCells = c(100, 500, 1000, 2000, 3000),
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column() %>%
  mutate(
    seed = 3000 + rowid,
    file_name = paste0(label,"_", batchCells, "_sim", "_", rep)
  )

# Changing number of clusters.
num_clusters_sims <- expand_grid(
  label = "num_clusters",
  data_id = "lawlor",
  de.facLoc = 2,
  n_clus = seq(10, 30, by = 10),
  rep = 1:2,
  batchCells = 2000
) %>%
  rowwise() %>%
  mutate(group.prob = list(rep(1 / n_clus, n_clus))) %>%
  ungroup() %>%
  rowid_to_column() %>%
  mutate(
    seed = 4000 + rowid,
    file_name = paste0(label,"_", n_clus, "_sim", "_", rep)
  )

# Changing proportion of cells in the cluster of interest (Group/cluster 1)
proportion_sims <- expand_grid(
  label = "prop",
  data_id = "lawlor",
  de.facLoc = 2,
  n_clus = 10,
  p = seq(0.01, 0.06, by = 0.02),
  rep = 1:2,
  batchCells = 2000
) %>%
  rowwise() %>%
  mutate(group.prob = list(c(p, rep((1 - p) / (n_clus - 1), n_clus - 1)))) %>%
  ungroup() %>%
  rowid_to_column() %>%
  mutate(
    seed = 5000 + rowid,
    file_name = paste0(label,"_", p, "_sim", "_", rep)
  )

# Changing de.facLoc parameter.
de_facloc_sims <- expand_grid(
  label = "de_facloc",
  data_id = "lawlor",
  de.facLoc = c(0.2, 0.5, 1, 2),
  rep = 1:2,
  batchCells = 2000,
  n_clus = 5,
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column %>%
  mutate(
    seed = 6000 + rowid,
    file_name = paste0(label, "_", de.facLoc, "_sim_", rep)
  )

mean_sims <- expand_grid(
  label = "mean",
  data_id = "lawlor",
  de.facLoc = 2,
  mean.shape = 1,
  mean.rate = c(4, 3, 2),
  rep = 1:2,
  batchCells = 2000,
  n_clus = 5,
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column %>%
  mutate(
    seed = 7000 + rowid,
    file_name = paste0(label, "_", mean.shape, "_", mean.rate, "_sim_", rep)
  )

all_sim_pars <- list(
  standard_sims,
  null_sims,
  num_cells_sims,
  num_clusters_sims,
  proportion_sims,
  de_facloc_sims,
  mean_sims
)

sim_pars <- flatten(map(all_sim_pars, ~ transpose(.x, .names = .x$file_name)))
sim_ids <- names(sim_pars)

sim_paths <- paste0(here::here(config$sim_pars_folder, sim_ids), ".json")
names(sim_paths) <- sim_ids

# Write all the parameters of each simulation to file.
# The checking of previous parameters is needed so that we don't constantly
# make the files newer, breaking Snakemake.
# FIXME: How can we clean up out of date parameter files?
for (id in sim_ids) {
  new <- sim_pars[[id]]
  path <- sim_paths[[id]]

  if (file.exists(path)) {

    old <- jsonlite::fromJSON(path)
    # Account for small floating point differences.
    if (!isTRUE(all.equal(old, new, tolerance = 1e-2))) {
      write(jsonlite::toJSON(new, null = "null"), path)
    }

  } else {
    write(jsonlite::toJSON(new, null = "null"), path)
  }
}

