reformat_found_mgs <- function(output, top_n = NULL, ...) {
  out <- switch(output$pars$method,
    "scran" = reformat_found_mgs_scran(output, top_n = top_n, ...),
    "seurat" = reformat_found_mgs_seurat(output, top_n = top_n, ...),
    stop("Invalid method", call. = FALSE)
  )
  out
}

reformat_found_mgs_seurat <- function(output, top_n) {

  stopifnot(output$pars$method == "seurat")

  res <- output$result
  mgs <- dplyr::group_by(res, cluster)
  is_roc <- output$pars$test.use == "roc"

  if (!is.null(top_n)) {
    if (is_roc) {
      mgs <- dplyr::arrange(mgs, desc(myAUC), desc(avg_log2FC))
    } else {
      mgs <- dplyr::arrange(mgs, p_val, desc(avg_log2FC))
    }
    mgs <- dplyr::slice(mgs, 1:top_n)
  }

  if (!is_roc) {
    mgs <- dplyr::select(mgs, p_value = p_val, p_value_adj = p_val_adj,
                         cluster, score = avg_log2FC, gene = gene)
  } else {
    mgs <- dplyr::select(mgs, cluster, score = myAUC, gene = gene)
    mgs <- dplyr::mutate(mgs, p_value = NA)
    mgs <- dplyr::mutate(mgs, p_value_adj = NA)
  }

  mgs <- dplyr::group_split(mgs)
  # Get rid of list_of.
  mgs <- as.list(mgs)
  # FIXME: Can this be made more robust?
  names(mgs) <- levels(res$cluster)
  mgs
}

reformat_found_mgs_scran <- function(output, top_n) {

  stopifnot(output$pars$method == "scran")

  res <- output$result
  n_clusters <- length(res)

  mgs <- lapply(res, function(x) {

    # Get rid of S4 DataFrame class and coerce to a tibble.
    x <- as.data.frame(x)
    x <- tibble::rownames_to_column(x, var = "gene")
    x <- tibble::as_tibble(x)

    # Perform optional filtering.
    if (!is.null(top_n)) {
      if (output$pars$pval.type == "any") {
        # FIXME: Think more about this heuristic.
        # Empirically, this produces approximately the right number of genes.
        # Generally, it gives slightly too many genes.
        x <- filter(x, Top < (top_n / n_clusters) * 2)
      } else {
        x <- dplyr::arrange(x, p.value)
        x <- dplyr::slice(x, 1:top_n)
      }
    }

    if (output$pars$test.type == "wilcox") {
      score_col <- "summary.AUC"
    } else {
      score_col <- "summary.logFC"
    }

    # TODO: Double check that the FDR is only the adjusted p-value.
    if (output$pars$pval.type == "any") {

      # We include Top as well when pval.type == "any" even though it is not
      # a score.

      # FIXME: What do the p values represent when pval.type == "any"?
      x <- select(x, p_value = p.value, score = !!score_col,
                  p_value_adj = FDR, gene = gene, top = Top)
    } else {
      x <- select(x, p_value = p.value, score = !!score_col,
                  p_value_adj = FDR, gene = gene)

    }

    x
  })

  mgs
}

# SUPERSEDED

#' Get the top genes from a marker gene selection method
#'
#' @param output The output from a supported marker gene selection method
#' @param n The number of top marker genes to return for each cluster
#' @param ... Other arguments passed to the specific functions for each method
#'
#' @return A list of the top `n` marker genes for each cluster.
#'
#' @details The currently supported marker gene selection methods are:
#'   * `scran::FindMarkers`
#'   * `Seurat::FindAllMarkers`
#'
get_top_genes <- function(output, n, ...) {

  out <- switch(
    output$pars$method,
    "scran" = get_top_genes_scran(output, n, ...),
    "seurat" = get_top_genes_seurat(output, n, ...),
    stop("Invalid method", call. = FALSE)
  )

  out
}

#' Get the top n genes from Seurat's marker gene output
#'
#' @param output The output from the `Seurat::FindAllMarkers` function.
#' @param n The number of top genes to select.
#'
#' @return A list of the top `n` marker genes for each cluster.
#'
#' @details In order to break ties in the sorting of p-values (which may occur
#'   when the p-values are all exactly 0) the genes are also sorted by
#'   `"avg_logFC"` (largest first).
#'
#'   When Seurat uses the `"roc"` test p-values are not generated. In this case
#'   the genes are ranked by their AUC values.
#'
get_top_genes_seurat <- function(output, n) {

  stopifnot(output$pars$method == "seurat")

  genes <- output$result
  top_genes <- dplyr::group_by(genes, cluster)

  # There are *no* p-values calculated for the ROC test.
  if (output$pars$test.use == "roc") {
    top_genes <- dplyr::arrange(top_genes, myAUC, desc(avg_log2FC))
  } else {
    top_genes <- dplyr::arrange(top_genes, p_val, desc(avg_log2FC))
  }

  top_genes <- dplyr::slice(top_genes, 1:n)
  top_genes <- dplyr::group_split(top_genes)

  out <- lapply(top_genes, function(x) x$gene)
  names(out) <- levels(genes$cluster)
  out
}

#' Get the top n genes from scran's marker gene output
#'
#' @param output The output from the `scran::findMarkers` function.
#' @param n The number of top genes to select.
#' @param any_method The method for selecting the top genes used when
#'   `pval.type = "any"`.
#'
#' @return A list of the top `n` marker genes for each cluster.
#'
#' @details When `pval.type = any` (the default) scran recommends choosing the
#'   marker genes by filtering on the `Top` column of the output. From the
#'   OSCA book: "The set of genes with Top <= x is the union of the top x genes
#'   (ranked by p-value) from each pairwise comparison". Selecting any given
#'   value of Top to filter on will therefore not guarantee that exactly
#'   n genes will be returned. Currently, we apply a rought heuristic that
#'   selects `Top` so that the number of returned genes is approximately `n`.
#'   This will definitely need to be refined in the future. Another option
#'   would be to use the returned p-value in this case which is calculated by
#'   applying Simes’ method to the pairwise p-values pairwise comparison.
#'
get_top_genes_scran <- function(output, n, any_method = "top_heuristic") {

  stopifnot(output$pars$method == "scran")
  stopifnot(any_method == "top_heuristic")

  genes <- output$result
  n_clusters <- length(genes)

  top_genes <- lapply(genes, function(x) {

    x <- as.data.frame(x)

    if (output$pars$pval.type == "any") {

      # FIXME: Think more about this heuristic.
      # Empirically, this produces approximately the right number of genes.
      if (any_method == "top_heuristic") {
        out <- filter(x, Top < (n / n_clusters) * 2)
      }

    } else {
      out <- dplyr::arrange(x, p.value)
      out <- dplyr::slice(x, 1:n)
    }

    rownames(out)
  })

  top_genes
}

#' Extract the indexes of DE genes from a splatter simulated
#' SingleCellExperiment object
#'
#' @param sce A SingleCellExperiment object create using by a splatter
#'   simulation.
#'
#' @return A list of the DE genes in each group.
#'
extract_de_inds <- function(sce) {
  stopifnot(is(sce, "SingleCellExperiment"))

  n_groups <- length(unique(colData(sce)$Group))
  col_names <- paste0("DEFacGroup", 1:n_groups)
  data <- rowData(sce)[, col_names]

  out <- lapply(data, function(x) which(x != 1))

  names(out) <- paste0("group_", 1:n_groups)
  out
}
